from sqlalchemy import *
from sqlalchemy.orm import *
from sqlalchemy.ext.declarative import declared_attr
from ..types import PollMood
from sqlalchemy.dialects.postgresql import UUID


class PollVote:
    __tablename__ = "pollvotes"

    @declared_attr
    def caster_id(self):
        return Column(Integer, ForeignKey("users.uid"), primary_key=True)

    @declared_attr
    def caster(self):
        return relationship("User", backref=backref("poll_votes_cast"))

    @declared_attr
    def poll_id(self):
        return Column(UUID(as_uuid=True), ForeignKey("polls.id"), primary_key=True)

    @declared_attr
    def poll(self):
        return relationship("Poll", backref=backref("votes"))

    @declared_attr
    def posted(self):
        return Column(DateTime, nullable=False)

    @declared_attr
    def vote(self):
        return Column(Enum(PollMood), nullable=False, default=PollMood.NEUTRAL)



from typing import *
import datetime
import uuid
from royalnet.utils import *
from royalnet.constellation.api import *
from ..tables import Poll


class ApiPollsCreate(ApiStar):
    path = "/api/polls/create/v1"

    summary = "Create a new poll."

    parameters = {
        "question": "The question to ask in the poll.",
        "description": "A longer Markdown-formatted description.",
        "expires": "A ISO timestamp of the expiration date for the poll.",
    }

    requires_auth = True

    methods = ["POST"]

    tags = ["polls"]

    async def api(self, data: ApiData) -> JSON:
        PollT = self.alchemy.get(Poll)

        poll = PollT(
            id=uuid.uuid4(),
            creator=await data.user(),
            created=datetime.datetime.now(),
            expires=datetime.datetime.fromisoformat(data["expires"]) if "expires" in data else None,
            question=data["question"],
            description=data.get("description"),
        )

        data.session.add(poll)
        await data.session_commit()

        return poll.json()



from starlette.requests import Request
from starlette.responses import *
from royalnet.constellation import *
from royalnet.utils import *
from ..tables import *
import uuid
from royalnet.constellation.api import *


class ApiWikiGetStar(ApiStar):
    path = "/api/wiki/get/v1"

    summary = "Get information about a specific wiki page."

    parameters = {
        "id": "The id of the wiki page to get information for."
    }

    tags = ["wiki"]

    async def api(self, data: ApiData) -> dict:
        wikipage_id_str = data["id"]
        try:
            wikipage_id = uuid.UUID(wikipage_id_str)
        except (ValueError, AttributeError, TypeError):
            raise InvalidParameterError("'id' is not a valid UUID.")
        wikipage: WikiPage = await asyncify(data.session.query(self.alchemy.get(WikiPage)).get, wikipage_id)
        if wikipage is None:
            raise NotFoundError("No such page.")
        return wikipage.json_full()



from typing import *
import datetime
from royalnet.utils import *
from royalnet.constellation.api import *
from ..tables import Poll
import uuid


class ApiPollsGet(ApiStar):
    path = "/api/polls/get/v1"

    summary = "Get the poll with a specific id."

    parameters = {
        "uuid": "The UUID of the poll to get.",
    }

    requires_auth = True

    tags = ["polls"]

    async def api(self, data: ApiData) -> JSON:
        PollT = self.alchemy.get(Poll)

        try:
            pid = uuid.UUID(data["uuid"])
        except (ValueError, AttributeError, TypeError):
            raise InvalidParameterError("'uuid' is not a valid UUID.")

        poll: Poll = await asyncify(data.session.query(PollT).get, pid)
        if poll is None:
            raise NotFoundError("No such page.")

        return poll.json()



from typing import *
from .lazyplay import LazyplayCommand


class LazysoundcloudCommand(LazyplayCommand):
    name: str = "lazysoundcloud"

    aliases = ["lsc"]

    description: str = "Cerca un video su SoundCloud e lo aggiunge (lazy) alla coda della chat vocale."

    syntax = "{ricerca}"

    async def get_urls(self, args):
        return [f"scsearch:{args.joined()}"]



from typing import *
import steam
import requests.exceptions
import royalnet.commands as rc
import royalnet.utils as ru
import royalnet.backpack.tables as rbt

from ..tables import Steam


class SteamGame:
    def __init__(self,
                 appid=None,
                 name=None,
                 playtime_forever=None,
                 img_icon_url=None,
                 img_logo_url=None,
                 has_community_visible_stats=None,
                 playtime_windows_forever=None,
                 playtime_mac_forever=None,
                 playtime_linux_forever=None,
                 playtime_2weeks=None):
        self.appid = appid
        self.name = name
        self.playtime_forever = playtime_forever
        self.img_icon_url = img_icon_url
        self.img_logo_url = img_logo_url
        self.has_community_visible_stats = has_community_visible_stats
        self.playtime_windows_forever = playtime_windows_forever
        self.playtime_mac_forever = playtime_mac_forever
        self.playtime_linux_forever = playtime_linux_forever
        self.playtime_2weeks = playtime_2weeks

    def __hash__(self):
        return self.appid

    def __eq__(self, other):
        if isinstance(other, SteamGame):
            return self.appid == other.appid
        return False

    def __str__(self):
        return self.name

    def __repr__(self):
        return f"<{self.__class__.__qualname__} {self.appid} ({self.name})>"


class SteammatchCommand(rc.Command):
    name: str = "steammatch"

    description: str = "Vedi quali giochi hai in comune con uno o più membri!"

    syntax: str = "{royalnet_username}+"

    def __init__(self, interface: rc.CommandInterface):
        super().__init__(interface)
        if "Steam" not in self.config or "web_api_key" not in self.config["Steam"]:
            raise rc.ConfigurationError("[c]Steam.web_api_key[/c] config option is missing!")
        self._api = steam.WebAPI(self.config["Steam"]["web_api_key"])

    async def run(self, args: rc.CommandArgs, data: rc.CommandData) -> None:
        users = []

        author = await data.get_author(error_if_none=True)
        users.append(author)

        for arg in args:
            user = await rbt.User.find(self.alchemy, data.session, arg)
            users.append(user)

        if len(users) < 2:
            raise rc.InvalidInputError("Devi specificare almeno un altro utente!")

        shared_games: Optional[set] = None
        for user in users:
            user_games = set()
            if len(user.steam) == 0:
                raise rc.UserError(f"{user} non ha un account Steam registrato!")
            for steam_account in user.steam:
                steam_account: Steam
                try:
                    response = await ru.asyncify(self._api.IPlayerService.GetOwnedGames,
                                                 steamid=steam_account._steamid,
                                                 include_appinfo=True,
                                                 include_played_free_games=True,
                                                 appids_filter=0)
                except requests.exceptions.HTTPError:
                    raise rc.ExternalError(f"L'account Steam di {user} è privato!")
                games = response["response"]["games"]
                for game in games:
                    user_games.add(SteamGame(**game))
            if shared_games is None:
                shared_games = user_games
            else:
                shared_games = shared_games.intersection(user_games)

        message_rows = [f"🎮 Giochi in comune tra {ru.andformat([str(user) for user in users], final=' e ')}:"]
        for game in sorted(list(shared_games), key=lambda g: g.name):
            message_rows.append(f"- {game}")

        message = "\n".join(message_rows)
        await data.reply(message)



from typing import *
import royalnet.commands as rc
import aiohttp
import urllib.parse

from .lazyplay import LazyplayCommand


class LazypeertubeCommand(LazyplayCommand):
    name: str = "lazypeertube"

    aliases = ["lpt", "lazyroyaltube", "lrt"]

    description: str = "Cerca un video su RoyalTube e lo aggiunge (lazy) alla coda della chat vocale."

    syntax = "{ricerca}"

    async def get_urls(self, args):
        search = urllib.parse.quote(args.joined(require_at_least=1))
        async with aiohttp.ClientSession() as session:
            async with session.get(self.config["Peertube"]["instance_url"] +
                                   f"/api/v1/search/videos?search={search}") as response:
                j = await response.json()
        if j["total"] < 1:
            raise rc.InvalidInputError("Nessun video trovato.")
        return [f'{self.config["Peertube"]["instance_url"]}/videos/watch/{j["data"][0]["uuid"]}']
